import os, sys, json, time, datetime, requests
from dateutil.tz import tzutc
import __main__


def __setServer__(local):
    credDir = os.path.expanduser('~') + os.sep + '.skolo' + os.sep
    if local:
        credFile = credDir + 'skolo.local.credentials'
        server = 'http://localhost:5000'
    else:
        credFile = credDir + 'skolo.credentials'
        try:
            server = open(os.path.expanduser('~') + os.sep + '.skolo' + os.sep + 'config').readlines()[0].split('server=')[-1]
        except:
            server = 'https://skolocfd.com'
    return server, credFile


def __respOK__(response):
    return 'HTTPStatusCode' in response and response['HTTPStatusCode'] == 200 and not response['errors']


class conn():
    def __init__(self, useLocal=None):
        self.__reqSession__ = requests.Session()
        
        self.__server__, credFile = __setServer__(useLocal)
        
        if not os.path.exists(credFile):
            self._exc(401, 'Missing credentials. For information on generating valid credentials visit the link provided.', link=self.__server__ + '/docs?topic=Api#Configuring%20Credentials')
            self.__cred__ = 'None'
        else:
            self.__cred__ = open(credFile, 'r').readlines()[0]
        self._checkVersion()
        
        
    def _clearLine(self, text):
        if hasattr(self, 'verbose') and self.verbose:
            if hasattr(self, 'lastPrintLine') and self.lastPrintLine == text:
                return
            print(text)
            self.lastPrintLine = text
            return
        sys.stdout.write("\033[K") #clear line 
        print(text.replace('\n',''), end='\r', flush=True)
        
    
    def _checkVersion(self, force=False):
        # Check for package updates
        verFile = os.path.expanduser('~') + os.sep + '.skolo' + os.sep + 'skolo.versionCheck'
        import skolo
        if not os.path.exists(verFile) or force or datetime.datetime.fromtimestamp(os.path.getmtime(verFile)) + datetime.timedelta(days=7) < datetime.datetime.now():
            try:
                vMax = max(requests.get('https://pypi.org/pypi/skolo/json').json()['releases'].keys())
            except:
                vMax = skolo.__version__
            if skolo.__version__ != vMax:
                print('\nALERT: skolo-' + skolo.__version__ + ' is installed, but skolo-' + vMax + ' is available:\n    python -m pip install --upgrade skolo\n')
            with open(verFile, 'w') as f:
                pass
        
    
    def _genResp(self, HTTPStatusCode=200, errors=[], warnings=[], messages=[], link=''):
        return {k:v for k,v in locals().items() if k != 'self'}
        
        
    def _exc(self, code, errMsg, link='' ):
        if type(errMsg) ==  str:
            errMsg = [errMsg]
        
        errResp = self._genResp(code, errors=errMsg, link=link)
        if hasattr(__main__, '__file__') and ('cliparser' in __main__.__file__.lower() or os.path.basename(__main__.__file__) == 'skolo'):
            print(json.dumps(errResp, indent=4, sort_keys=True))
            sys.exit(-1)
        else:
            print('    ERROR ' + str(code) + ': ' + ' '.join(errMsg))
            print('      ' + link)
            return errResp
            #raise Exception(errMsg)
        
    
    def _buildResp(self, r):
        resp = {}
        if r.status_code == 401:
            resp = self._exc(r.status_code, 'Invalid credentials. For information on generating valid credentials visit the link provided.', link=self.__server__ + '/docs?topic=Api#Configuring%20Credentials')
        elif r.status_code == 403:
            resp = self._exc(r.status_code, 'Expired credentials. Please generate a new authentication token at the link provided.', link=self.__server__ + '/docs?topic=Api#Configuring%20Credentials')
        elif r.status_code == 404:
            resp = self._exc(r.status_code, r.text)
        elif r.status_code > 399:
            resp = self._exc(r.status_code, 'Unhandled server error - Please contact us.')
            
        if '[' not in r.text and '{' not in r.text:
            resp.update({'messages':[r.text]})
        else:
            resp = json.loads(r.text)
        resp.update({'HTTPStatusCode':r.status_code})
        return resp, r
        
        
    def _http(self, url, req, json=None, data=None, files=None):
        args = {'url':url, 'json':json, 'data':data, 'files':files, 'auth':(self.__cred__, 'none')}
        args = {k:v for k,v in args.items() if v != None}
        if 'files' not in args:
            args['headers'] = {'Content-Type': 'application/json'}
            
        try:
            return self._buildResp(getattr(self.__reqSession__, req)(**args))
        except requests.exceptions.ConnectionError:
            try:
                a = self.__reqSession__.get('http://www.google.com')
            except requests.exceptions.ConnectionError:
                resp = self._exc(500, 'Connection refused. It appears your internet connection is down.')
            resp = self._exc(500, 'Connection refused. It appears the Skolo server is unreachable.')
            return resp, None
    
    
    def _get(self, url, json={}):
        return self._http(url, 'get', json)
    
    
    def _post(self, url, json={}):
        return self._http(url, 'post', json)
    
    
    def _uploadFile(self, url, file, data):
        fd = {'file': ''}
        with open(file, 'rb') as fd['file']:
            return self._http(url, 'post', None, data, fd)
    
    
    def _buildUrl(self, pre, suf='', filters=True):
        if not filters:
            return self.__server__ + pre    
        return self.__server__ + pre + self._filterUrl(self.__filters__) + suf
    
    
    def _filterUrl(self, filters, ignore=[]):
        self.__filters__ = filters
        t = ''
        for k,v in filters.items():
            if k in ignore:
                continue
            t += '?' + k + '=' + v
        return t
    
    
    def checkExist(self):
        """
        Checks if the Skolo instance (Project, Run, Orientation) exists on the server

        Returns
        -------
        response : dict
            Skolo HTTP response dictionary, see https://www.skolocfd.com/docs?topic=Api
            If the HTTPStatusCode in the dict is 200, the Project/Run/Orientaton exists
        """
        url = self._buildUrl('/api/exists')
        return self._get(url)[0]
    
    
    def _listChildren(self):
        url = self._buildUrl('/api/list')
        return self._get(url)[0]
    
    
    def _taskWatch(self, resp, cmd, queue, mins):
        watchUrl = self._buildUrl('/getStatus/' + resp['jobID'] + '/' + queue + '?fromApi=true', filters=False   )
        t, dt, tMax, status = 0, 1, 60*mins, 'queueing'
        while t < tMax and status not in ['finished', 'failed']:
            watch, w = self._get(watchUrl)
            status = watch['task']['status']
            
            if 'log' in watch['task']['meta'] and len(watch['task']['meta']['log']) > 0:
                lines = [line for line in watch['task']['meta']['log'] if not line.startswith(' ')]
                if lines != []:
                    self._clearLine(lines[-1])
            elif status != 'queueing':
                self._clearLine(cmd + ' job is ' + status)
            
            if 'geomCheckRhs' in watch['task']['meta']:
                watch['geomCheckRhs'] = watch['task']['meta']['geomCheckRhs']
            
            time.sleep(dt)
            t += dt
        print()
        watch['taskLog'] = watch['task']['meta']['log']
        watch['link'] = self._buildUrl('/case?proj=', '?step=upload')
        del(watch['task'])
        
        return watch



# Generic account connection
class acct(conn):
    def __init__(self, args):
        kD = {'project':'proj', 'run':'case', 'orien':'rh'}
        self.__filters__={}
        for key in ['project', 'proj', 'run', 'case', 'orien', 'rh']:
            if key in kD:
                target = kD[key]
            else:
                target = key
            if hasattr(args, key) and getattr(args, key):
                self.__filters__[target] = getattr(args, key)
        super(acct, self).__init__(args.local)
        self.checkExist()


class Project(conn):
    def __init__(self, proj, useLocal=False):
        """
        Initialize a connection to a Skolo Project

        Inputs
        ----------
        proj : string
            Skolo project name
        useLocal : bool
            connect to local rather than remote server

        Returns
        -------
        run : object
            An instance of this class
        """
        
        self.proj = proj
        self.__filters__ = {'proj':proj}
        super(Project, self).__init__(useLocal)
        self.checkExist()
        
        
    def listRuns(self):
        """
        Returns a list of all runs in the project
        """
        resp = self._listChildren()
        
        return resp['runs'] if 'runs' in resp else []



class Run(conn):
    def __init__(self, proj, run, useLocal=False):
        """
        Initialize a connection to a Skolo Run

        Inputs
        ----------
        proj : string
            Skolo project name
        run : string
            Skolo run number
        useLocal : bool
            connect to local rather than remote server

        Returns
        -------
        run : object
            An instance of this class
        """
        
        self.proj = proj
        self.run = run
        self.__filters__ = {'proj':proj, 'case':run}
        super(Run, self).__init__(useLocal)
        self.checkExist()
        
        
    def listOrientations(self):
        """
        Returns a list of all orientations for a given run; no inputs.
        """
        resp = self._listChildren()
        return resp['orienList'] if 'orienList' in resp else []
    
    
    def createNew(self, comment, topic='empty', copyGeometry=True, copyOrientations=True, copySettings=True, copyPostPro=True):
        """
        Function that creates a new run, based on the current run.

        Inputs
        ----------
        comment : string
            descriptive comment for the run
        topic : string
            descriptine topic for the run - equivalent to "topic" in the Skolo GUI
        copyGeometry : bool
            copy all geometry from the baseline run
        copyOrientations : bool
            copy all orientations from the baseline run
        copySettings : bool
            copy all run settings from the baseline run
        copyPostPro : bool
            copy all postPro templates from the baseline run

        Returns
        -------
        resp : dict
            Skolo HTTP response dictionary, see https://www.skolocfd.com/docs?topic=Api
        """
        
        jsonData = {'topic':str(topic), 'comment':comment, 'geom':copyGeometry, 'settings':copySettings, 'copyRhs':copyOrientations, 'post':copyPostPro, 'cadID':''}
        url = self._buildUrl('/api/newCase')
        resp, r = self._post(url, jsonData)
        return self._taskWatch(resp, 'newCase', 'berdQ', 3)

        
        
# Connection to specific proj:run:orientation
class Orientation(conn):
    def __init__(self, proj, run, orien, useLocal=False):
        """
        Initialize a connection to a Skolo Orientation within a Run

        Inputs
        ----------
        proj : string
            Skolo project name
        run : string
            Skolo run number
        orien : string
            Skolo orientation
        useLocal : bool
            connect to local rather than remote server

        Returns
        -------
        run : object
            An instance of this class
        """
        self.proj = proj
        self.run = run
        self.orien = orien
        self.useLocal = useLocal
        self.__filters__ = {'proj':proj, 'case':run, 'rh':orien}
        super(Orientation, self).__init__(useLocal)
        self.checkExist()
    
    
    def geomCompare(self):
        """
        Submits a geometry comparison job vs. the baseline

        Returns
        -------
        resp : dict
            Skolo HTTP response dictionary, see https://www.skolocfd.com/docs?topic=Api
        """

        cmd = 'geomCompare'
        suf = '?fromApi=true?fullCheck=false?debug=false'
        
        url = self._buildUrl('/compareGeom', suf)
        jsonData = {}
        resp, r = self._post(url, jsonData)
        
        # Catch case where server chooses not to run geomPrep
        if 'jobID' not in resp and 'rhError' in resp:
            resp['errors'] = [resp['rhError']]
            del resp['rhError']
            resp['HTTPStatusCode'] = 400
            return resp
        print(cmd.capitalize() + ' job is queueing', end='\r')
        
        # Watch job status
        return self._taskWatch(resp, cmd, 'geomQ', 5)
    
    
    def kinematics(self, force=False):
        """
        Submits kinematic model movement, and checks the resulting output geometry

        Inputs
        ----------
        force : bool
            forces re-running of kinematics for all rideheights whether needed or not - WARNING! overwrites files!

        Returns
        -------
        resp : dict
            Skolo HTTP response dictionary, see https://www.skolocfd.com/docs?topic=Api
        """
        
        resp = self._geomTask('kinematics', force=force)
        
        # geomCheck all the output rhs
        print("THIS IS THE RESP", resp)
        print(resp['geomCheckRhs'])
        for rh in resp['geomCheckRhs']:
            print("RIDEHEIGHT", rh)
            orien = Orientation(self.proj, self.run, rh, self.useLocal)
            orien.submit('setup')
        
        return resp
    
    
    def geomPrep(self, force=False):
        """
        Submits geomPrep, and checks the resulting output geometry

        Inputs
        ----------
        force : bool
            forces re-running of all geomPrep on all CAD files - WARNING! overwrites files!

        Returns
        -------
        resp : dict
            Skolo HTTP response dictionary, see https://www.skolocfd.com/docs?topic=Api
        """
        
        resp = self._geomTask('geomPrep', force=force)
        
        # geomCheck the output files
        self.submit('setup')
        
        return resp
    
    
    def _geomTask(self, cmd, force=False):
        """
        Submits geomPrep/kinematics
        
        Optional arguments:
            force:      forces re-running of all geomPrep on all CAD files - WARNING! overwrites files!
        """
        if self.orien != 'geomConstruction':
            return self._genResp(errors=['GeomPrep and Kinematics may only be run from the construion geometry orientation (geomConstruction)'])
        
        # Submit job
        suf = '?fromApi=true?rhSel=geomConstruction?fcnToRun=' + cmd
        if force:
            suf += '?forceReRun=true'
        
        
        url = self._buildUrl('/runGeomPrep', suf)
        
        print("URL", url)
        
        jsonData = {'files':[]}
        resp, r = self._post(url, jsonData)
        
        # Catch case where server chooses not to run geomPrep
        if 'jobID' not in resp and 'rhError' in resp:
            resp['errors'] = [resp['rhError']]
            del resp['rhError']
            resp['HTTPStatusCode'] = 400
            return resp
        print(cmd.capitalize() + ' job is queueing', end='\r')
        
        # Watch job status
        return self._taskWatch(resp, cmd, 'ansaQ', 20)
    
    
    def submit(self, cmd, costLimit=None):
        """
        Submits the current orientation with the desired command

        Inputs
        ----------
        cmd : string
            the command to run, which can be: setup, mesh, post, or submit (submit includes mesh, solve, post)
        costLimit : integer
            Specify a limit to acceptable estimated cost, above which job submission will be aborted

        Returns
        -------
        resp : dict
            Skolo HTTP response dictionary, see https://www.skolocfd.com/docs?topic=Api
        """
        
        if self.orien == 'geomCommon':
            return self._genResp(errors=['geomCommon is not a valid orientation. Such an orientation may only used by the upload command'])
        
        if cmd == None:
            cmd = 'submit'
        
        # Submit job
        suf = '?fromApi=true'
        if costLimit:
            float(costLimit)
            suf += '?costLimit=' + str(int(costLimit))
        
        url = self._buildUrl('/runBerd', suf)
        jsonData = {'cmd':cmd, 'rhSel':self.__filters__['rh']}
        resp, r = self._post(url, jsonData)
        print(cmd.capitalize() + ' job is queueing', end='\r')
        
        # Watch job status
        return self._taskWatch(resp, cmd, 'berdQ', 3)
        
        
    def _describeFile(self, f):
        return {'upload':True, 'size':os.path.getsize(f), 'mtime':str(datetime.datetime.fromtimestamp(os.path.getmtime(f), tz=tzutc()))}
        
        
    def upload(self, folder, force=False):
        """
        Uploads geometry to the orientation

        Inputs
        ----------
        folder : string
            the file or folder to upload.
            file - the file will always be uploaded
            folder - step, STL or CATIA files in the folder which are NOT already up-to-date on the server will be uploaded (speeds things up)
        force : bool
            force the upload of all files in the folder, even if they are already up-to-date on the server

        Returns
        -------
        resp : dict
            Skolo HTTP response dictionary, see https://www.skolocfd.com/docs?topic=Api
        """
        
        t0 = time.time()
        from concurrent.futures import ThreadPoolExecutor
        exts = ('.stl', '.stl.gz', '.vtp', '.vtp.gz', '.stp', '.step', 'catpart', '3dxml')
        
        # Create list of files
        files, tSize, t0 = {}, 0, time.time()
        if os.path.isdir(folder):
            for item in sorted(os.listdir(folder), key=str.casefold):
                if item.endswith(exts):
                    ful = folder + os.sep + item
                    files[ful] = self._describeFile(ful)
        elif os.path.exists(folder):
            files[folder] = self._describeFile(folder)
            force = True
        else:
            self._exc('Specified file location is neither a file nor a folder: ' + folder)
        
        # See which files are actually out-of-date on server
        if not force:
            url = self._buildUrl('/api/needsUpload')
            respNeed, r = self._get(url, json=files)
            for f in files:
                if f in respNeed['fileInfo'] and not respNeed['fileInfo'][f]['upload']:
                    files[f]['upload'] = False
        
        # Parallel uploading
        resp = {'HTTPStatusCode':200}
        url = self._buildUrl('/upload', '?units=Meters')
        def parUpload(geom):
            resp, r = self._uploadFile(url, geom, files[geom])
            self._clearLine('    Uploading: ' + geom)
            return (geom, resp)

        pool = ThreadPoolExecutor(max_workers=6)
        filesToUpload = sorted([item for item in files if force or respNeed['fileInfo'][item]['upload']], key=str.casefold)
        for geom in pool.map(parUpload, filesToUpload):
            if geom[1]['HTTPStatusCode'] == 200:
                files[geom[0]] = 'Upload successful'
                tSize += os.path.getsize(geom[0])
        self._clearLine('')
        
        if len(files) == 0:
            warnings = ['Zero suitable files (' + ', '.join(exts) + ') found at destination: ' + os.path.abspath(folder)]
        else:
            warnings = []

        resp = {'HTTPStatusCode':resp['HTTPStatusCode'], 'messages':['Uploaded %.1f Mb from %i files in %.1f sec.' % (tSize/(1024*1024), len(filesToUpload), time.time()-t0)], 'errors':[], 'warnings':warnings, 'fileInfo':files}
        resp['link'] = self._buildUrl('/case?proj=', '?step=upload')
        return resp


#!/usr/bin/env python
import os, sys, json, argparse, webbrowser
import skolo
from skolo.caseObjects import acct, __setServer__, __respOK__


def checkCred(local):
    server, credFile = __setServer__(local)
    if not os.path.exists(credFile):
        print('\nMust exit: credentials not found:', credFile)
        print('    This tool requires access to the Skolo platform, via credentialed tokens provided under your username')
        print('    Please visit the link below for instructions on downloading and installing these credentials')
        print('\n    ' + server + '/docs?topic=Api#Configuring%20Credentials\n')
        sys.exit()


def printResponse(resp, args):
    print(json.dumps(resp, indent=4, sort_keys=True))
    if hasattr(args, 'browser') and args.browser and 'link' in resp:
        webbrowser.open(resp['link'])


def LIST(args):
    resp = acct(args)._listChildren()
    printResponse(resp, args)


def upload(args):
    orien = skolo.Orientation(args.project, args.run, args.orientation, args.local)
    printResponse(orien.upload(args.folder, args.force), args)


def geom(args):
    orien = skolo.Orientation(args.project, args.run, args.orientation, args.local)
    calls = []
    
    # geomPrep requires a geomPrep and checkGeom call
    if args.command in ['geomPrep', 'all']:
        calls.append({'fcn':orien.geomPrep, 'args':(args.force,)})
    
    # kinematics requires a geomPrep and several checkGeom calls
    if args.command in ['kinematics', 'all']:
        calls.append({'fcn':orien.kinematics, 'args':(args.force,)})
    
    # GeomCompare is simple
    if args.command in ['geomCompare', 'all']:
        calls.append({'fcn':orien.geomCompare, 'args':(args.force,)})
    
    # Call each of the functions, exiting if at any point one of them fails
    moreCalls = []
    for call in calls:
        resp = call['fcn'](*call['args'])
        
        if resp['HTTPStatusCode'] != 200:
            print('Exiting due to server failure')
            return
        elif resp['errors']:
            print('Exiting due to errors')
            return
    
    return


def create(args):
    run = skolo.Run(args.project, args.baseline, args.local)
    inArgs = {k:v for k,v in vars(args).items() if k in ['comment', 'topic', 'noGeometry', 'noOrientations', 'noSettings', 'noPostPro']}
    
    kwargs = {}
    for key in inArgs:
        if key.startswith('no'):
            kwargs[key.replace('no', 'copy')] = not inArgs[key]
        else:
            kwargs[key] = inArgs[key]

    printResponse(run.createNew(**kwargs), args)


def submit(args):
    orien = skolo.Orientation(args.project, args.run, args.orientation, args.local)
    printResponse(orien.submit(args.execute, args.costLimit), args)


def version(args):
    import requests
    latest = max(requests.get('https://pypi.org/pypi/skolo/json').json()['releases'].keys())
    print('SkoloCFD CLI')
    print('  Current version:', skolo.__version__)
    print('  Latest version: ', latest)
    if latest != skolo.__version__:
        print('\n  You can upgrade with:\n    python -m pip install --upgrade skolo')


def main():
    # create the top-level parser
    parser = argparse.ArgumentParser(prog='skolo', description='skolo is the command line tool for the SkoloCFD cloud platform')
    parser.add_argument('-l', '--local', help='Connect to a Skolo server hosted locally', action='store_true')
    parser._positionals.title = 'Available commands'
    subparsers = parser.add_subparsers(help='--Command Description--')

    # create the parser for the "list" command
    parserList = subparsers.add_parser('list', help='LIST projects, runs, or orientations', description='LIST will display all exiting projects (default), runs (if given a project) or orientations (if given a run and project)')
    parserList.add_argument('-p', '--project', help='Specify a project to list all runs')
    parserList.add_argument('-r', '--run', help='Specify a run number to list orientations')
    parserList.set_defaults(func=LIST)
    parserList._positionals.title = 'Required arguments'

    # create the parser for the "create" command
    parserCreate = subparsers.add_parser('create', help='CREATE a new run', description='CREATE allows you to generate new runs referenced to a baseline.')
    parserCreate.add_argument('project', help='Specify the project')
    parserCreate.add_argument('baseline', help='Set the baseline run on which the new case is based')
    parserCreate.add_argument('comment', help='Description for the run')
    parserCreate.add_argument('-g', '--noGeometry', help='Do not copy geometry from the baseline run', action='store_true')
    parserCreate.add_argument('-t', '--topic', help='Tag for the run - useful for searching project tables')
    group = parserCreate.add_argument_group('optional overrides: by default, postPro templates, orientations, & settings from the baseline are copied to the new run. To override this one-by-one')
    group.add_argument('-no', '--noOrientations', help='Dont\' copy orientations from the baseline run.', action='store_true')
    group.add_argument('-np', '--noPostPro', help='Don\'t copy post-pro templates from the baseline run', action='store_true')
    group.add_argument('-ns', '--noSettings', help='Dont\' copy run settings from the baseline run.', action='store_true')
    parserCreate.add_argument('-b', '--browser', help='Open a browser tab for the new case upon completion.', action='store_true')
    parserCreate.set_defaults(func=create)
    parserCreate._positionals.title = 'Required arguments'

    # create the parser for the "upload" command
    parserUpload = subparsers.add_parser('upload', help='UPLOAD geometries to a run/orientation', description='UPLOAD allows you to upload a list or folder of files. All allowable filetypes (stl & stl.gz) will be uploaded. Other filetypes will be ignored.')
    parserUpload.add_argument('project', help='Specify the project')
    parserUpload.add_argument('run', help='Specify the run number')
    parserUpload.add_argument('orientation', help='Specify the orientation.\nSetting "geomCommon" as the orientation will upload to a directory shared by all current and future orientations.')
    parserUpload.add_argument('-f', '--folder', help='Folder or file to upload')
    parserUpload.add_argument('--force', help='By default, only files newer than any on the server are. This option forces a fresh upload.', action='store_true')
    parserUpload.add_argument('-b', '--browser', help='Open a browser tab for the new case upon completion.', action='store_true')
    parserUpload.set_defaults(func=upload)
    parserUpload._positionals.title = 'Required arguments'

    # create the parser for the "geom" command
    parserGeom = subparsers.add_parser('geom', help='Process GEOM for a run', description='GEOM allows you to process CAD, perform kinematics, compare to a baseline, or perform all 3 in sequence')
    parserGeom.add_argument('project', help='Specify the project')
    parserGeom.add_argument('run', help='Specify the run number')
    parserGeom.add_argument('orientation', help='Specify which orientation, most commonly geomConstruction')
    parserGeom.add_argument('command', help='Command to execute in a comma separated list.', choices=['geomPrep', 'kinematics', 'geomCompare', 'all'])
    parserGeom.add_argument('-b', '--browser', help='Open a browser tab to view results upon completion.', action='store_true')
    parserGeom.add_argument('-f', '--force', help='Forcibly re-run the command(s) specified. Warning: this may overwrite files!', action='store_true')
    parserGeom.set_defaults(func=geom)
    parserGeom._positionals.title = 'Required arguments'

    # create the parser for the "submit" command
    parserSubmit = subparsers.add_parser('submit', help='SUBMIT an orientation for solving', description='SUBMIT allows you to run the specified orientation.')
    parserSubmit.add_argument('project', help='Specify the project')
    parserSubmit.add_argument('run', help='Specify the run number')
    parserSubmit.add_argument('orientation', help='Specify which orientation')
    parserSubmit.add_argument('-e', '--execute', help='Command to execute. Default is "submit" Other options allow you to perform only setup, meshing, or post processing.', choices=['setup', 'mesh', 'submit', 'post'])
    parserSubmit.add_argument('-c', '--costLimit', help='Set a cost limit, above which the operation will not execute.', type=int)
    parserSubmit.add_argument('-b', '--browser', help='Open a browser tab for the new case upon completion.', action='store_true')
    parserSubmit.set_defaults(func=submit)
    parserSubmit._positionals.title = 'Required arguments'

    # create the parser for the "version" command
    parserSubmit = subparsers.add_parser('version', help='List current and latest version', description='VERSION lists the current and latest version')
    parserSubmit.set_defaults(func=version)
    parserSubmit._positionals.title = 'Required arguments'

    # Parse and run function
    args = parser.parse_args()
    
    # Check credentials are installed
    checkCred(args.local)
    
    # Run the chosen function
    if hasattr(args, 'func'):
        args.func(args)
    else:
        parser.print_help()


if __name__ == '__main__':
    main()


from setuptools import setup, find_packages

with open("README.md", "r") as readme_file:
    readme = readme_file.read()

requirements = ["requests>=2.20", 'python-dateutil>=2']

import os.path

def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()

def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")

setup(
    name="skolo",
    version=get_version("skolo/__init__.py"),
    author="Skolo, LLC",
    author_email="support@skolocfd.com",
    license = 'MIT License',
    description="A command line tool and python package for SkoloCFD",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://skolocfd.com/docs?topic=Api",
    py_modules = ['skoloCliParser'],
    packages=find_packages(),
    install_requires=requirements,
    classifiers=[
        "Programming Language :: Python :: 3.11",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
    ],
    entry_points = '''
        [console_scripts]
        skolo=skoloCliParser:main
    ''',
    python_requires='>=3.0',
)